@@ -4,35 +4,41 @@
 
 #include "device.h"
 
-#include <qbluetoothaddress.h>
-#include <qbluetoothdevicediscoveryagent.h>
-#include <qbluetoothlocaldevice.h>
-#include <qbluetoothdeviceinfo.h>
-#include <qbluetoothservicediscoveryagent.h>
+#include <QBluetoothDeviceInfo>
+#include <QBluetoothUuid>
+
 #include <QDebug>
-#include <QList>
-#include <QMetaEnum>
+#include <QMetaObject>
 #include <QTimer>
 
+#if QT_CONFIG(permissions)
+#include <QPermissions>
+
+#include <QGuiApplication>
+#endif
+
+using namespace Qt::StringLiterals;
+
 Device::Device()
 {
     //! [les-devicediscovery-1]
-    discoveryAgent = new QBluetoothDeviceDiscoveryAgent();
+    discoveryAgent = new QBluetoothDeviceDiscoveryAgent(this);
     discoveryAgent->setLowEnergyDiscoveryTimeout(25000);
     connect(discoveryAgent, &QBluetoothDeviceDiscoveryAgent::deviceDiscovered,
             this, &Device::addDevice);
-    connect(discoveryAgent, &QBluetoothDeviceDiscoveryAgent::errorOccurred, this,
-            &Device::deviceScanError);
-    connect(discoveryAgent, &QBluetoothDeviceDiscoveryAgent::finished, this, &Device::deviceScanFinished);
+    connect(discoveryAgent, &QBluetoothDeviceDiscoveryAgent::errorOccurred,
+            this, &Device::deviceScanError);
+    connect(discoveryAgent, &QBluetoothDeviceDiscoveryAgent::finished,
+            this, &Device::deviceScanFinished);
+    connect(discoveryAgent, &QBluetoothDeviceDiscoveryAgent::canceled,
+            this, &Device::deviceScanFinished);
     //! [les-devicediscovery-1]
 
-    setUpdate("Search");
+    setUpdate(u"Search"_s);
 }
 
 Device::~Device()
 {
-    delete discoveryAgent;
-    delete controller;
     qDeleteAll(devices);
     qDeleteAll(m_services);
     qDeleteAll(m_characteristics);
@@ -47,39 +53,52 @@ void Device::startDeviceDiscovery()
     devices.clear();
     emit devicesUpdated();
 
-    setUpdate("Scanning for devices ...");
     //! [les-devicediscovery-2]
     discoveryAgent->start(QBluetoothDeviceDiscoveryAgent::LowEnergyMethod);
     //! [les-devicediscovery-2]
 
     if (discoveryAgent->isActive()) {
+        setUpdate(u"Stop"_s);
         m_deviceScanState = true;
         Q_EMIT stateChanged();
     }
 }
 
+void Device::stopDeviceDiscovery()
+{
+    if (discoveryAgent->isActive())
+        discoveryAgent->stop();
+}
+
 //! [les-devicediscovery-3]
 void Device::addDevice(const QBluetoothDeviceInfo &info)
 {
-    if (info.coreConfigurations() & QBluetoothDeviceInfo::LowEnergyCoreConfiguration)
-        setUpdate("Last device added: " + info.name());
+    if (info.coreConfigurations() & QBluetoothDeviceInfo::LowEnergyCoreConfiguration) {
+        auto devInfo = new DeviceInfo(info);
+        auto it = std::find_if(devices.begin(), devices.end(),
+                               [devInfo](DeviceInfo *dev) {
+                                   return devInfo->getAddress() == dev->getAddress();
+                               });
+        if (it == devices.end()) {
+            devices.append(devInfo);
+        } else {
+            auto oldDev = *it;
+            *it = devInfo;
+            delete oldDev;
+        }
+        emit devicesUpdated();
+    }
 }
 //! [les-devicediscovery-3]
 
 void Device::deviceScanFinished()
 {
-    const QList<QBluetoothDeviceInfo> foundDevices = discoveryAgent->discoveredDevices();
-    for (auto &nextDevice : foundDevices)
-        if (nextDevice.coreConfigurations() & QBluetoothDeviceInfo::LowEnergyCoreConfiguration)
-            devices.append(new DeviceInfo(nextDevice));
-
-    emit devicesUpdated();
     m_deviceScanState = false;
     emit stateChanged();
     if (devices.isEmpty())
-        setUpdate("No Low Energy devices found...");
+        setUpdate(u"No Low Energy devices found..."_s);
     else
-        setUpdate("Done! Scan Again!");
+        setUpdate(u"Done! Scan Again!"_s);
 }
 
 QVariant Device::getDevices()
@@ -108,7 +127,7 @@ void Device::scanServices(const QString &address)
 
     for (auto d: std::as_const(devices)) {
         if (auto device = qobject_cast<DeviceInfo *>(d)) {
-            if (device->getAddress() == address ) {
+            if (device->getAddress() == address) {
                 currentDevice.setDevice(device->getDevice());
                 break;
             }
@@ -127,7 +146,7 @@ void Device::scanServices(const QString &address)
     m_services.clear();
     emit servicesUpdated();
 
-    setUpdate("Back\n(Connecting to device...)");
+    setUpdate(u"Back\n(Connecting to device...)"_s);
 
     if (controller && m_previousAddress != currentDevice.getAddress()) {
         controller->disconnectFromDevice();
@@ -138,7 +157,7 @@ void Device::scanServices(const QString &address)
     //! [les-controller-1]
     if (!controller) {
         // Connecting signals and slots for connecting to LE services.
-        controller = QLowEnergyController::createCentral(currentDevice.getDevice());
+        controller = QLowEnergyController::createCentral(currentDevice.getDevice(), this);
         connect(controller, &QLowEnergyController::connected,
                 this, &Device::deviceConnected);
         connect(controller, &QLowEnergyController::errorOccurred, this, &Device::errorReceived);
@@ -178,7 +197,7 @@ void Device::addLowEnergyService(const QBluetoothUuid &serviceUuid)
 
 void Device::serviceScanDone()
 {
-    setUpdate("Back\n(Service scan done!)");
+    setUpdate(u"Back\n(Service scan done!)"_s);
     // force UI in case we didn't find anything
     if (m_services.isEmpty())
         emit servicesUpdated();
@@ -210,7 +229,7 @@ void Device::connectToService(const QString &uuid)
         connect(service, &QLowEnergyService::stateChanged,
                 this, &Device::serviceDetailsDiscovered);
         service->discoverDetails();
-        setUpdate("Back\n(Discovering details...)");
+        setUpdate(u"Back\n(Discovering details...)"_s);
         //! [les-service-3]
         return;
     }
@@ -227,7 +246,7 @@ void Device::connectToService(const QString &uuid)
 
 void Device::deviceConnected()
 {
-    setUpdate("Back\n(Discovering services...)");
+    setUpdate(u"Back\n(Discovering services...)"_s);
     connected = true;
     //! [les-service-2]
     controller->discoverServices();
@@ -237,7 +256,7 @@ void Device::deviceConnected()
 void Device::errorReceived(QLowEnergyController::Error /*error*/)
 {
     qWarning() << "Error: " << controller->errorString();
-    setUpdate(QString("Back\n(%1)").arg(controller->errorString()));
+    setUpdate(u"Back\n(%1)"_s.arg(controller->errorString()));
 }
 
 void Device::setUpdate(const QString &message)
@@ -298,18 +317,17 @@ void Device::serviceDetailsDiscovered(QLowEnergyService::ServiceState newState)
 
 void Device::deviceScanError(QBluetoothDeviceDiscoveryAgent::Error error)
 {
-    if (error == QBluetoothDeviceDiscoveryAgent::PoweredOffError)
-        setUpdate("The Bluetooth adaptor is powered off, power it on before doing discovery.");
-    else if (error == QBluetoothDeviceDiscoveryAgent::InputOutputError)
-        setUpdate("Writing or reading from the device resulted in an error.");
-    else {
+    if (error == QBluetoothDeviceDiscoveryAgent::PoweredOffError) {
+        setUpdate(u"The Bluetooth adaptor is powered off, power it on before doing discovery."_s);
+    } else if (error == QBluetoothDeviceDiscoveryAgent::InputOutputError) {
+        setUpdate(u"Writing or reading from the device resulted in an error."_s);
+    } else {
         static QMetaEnum qme = discoveryAgent->metaObject()->enumerator(
                     discoveryAgent->metaObject()->indexOfEnumerator("Error"));
-        setUpdate("Error: " + QLatin1String(qme.valueToKey(error)));
+        setUpdate(u"Error: "_s + QLatin1StringView(qme.valueToKey(error)));
     }
 
     m_deviceScanState = false;
-    emit devicesUpdated();
     emit stateChanged();
 }
 
