@@ -1,41 +1,5 @@
-/****************************************************************************
- **
- ** Copyright (C) 2016 The Qt Company Ltd.
- ** Contact: https://www.qt.io/licensing/
- **
- ** This file is part of the QtBluetooth module of the Qt Toolkit.
- **
- ** $QT_BEGIN_LICENSE:LGPL$
- ** Commercial License Usage
- ** Licensees holding valid commercial Qt licenses may use this file in
- ** accordance with the commercial license agreement provided with the
- ** Software or, alternatively, in accordance with the terms contained in
- ** a written agreement between you and The Qt Company. For licensing terms
- ** and conditions see https://www.qt.io/terms-conditions. For further
- ** information use the contact form at https://www.qt.io/contact-us.
- **
- ** GNU Lesser General Public License Usage
- ** Alternatively, this file may be used under the terms of the GNU Lesser
- ** General Public License version 3 as published by the Free Software
- ** Foundation and appearing in the file LICENSE.LGPL3 included in the
- ** packaging of this file. Please review the following information to
- ** ensure the GNU Lesser General Public License version 3 requirements
- ** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
- **
- ** GNU General Public License Usage
- ** Alternatively, this file may be used under the terms of the GNU
- ** General Public License version 2.0 or (at your option) the GNU General
- ** Public license version 3 or any later version approved by the KDE Free
- ** Qt Foundation. The licenses are as published by the Free Software
- ** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
- ** included in the packaging of this file. Please review the following
- ** information to ensure the GNU General Public License requirements will
- ** be met: https://www.gnu.org/licenses/gpl-2.0.html and
- ** https://www.gnu.org/licenses/gpl-3.0.html.
- **
- ** $QT_END_LICENSE$
- **
- ****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 package org.qtproject.qt.android.bluetooth;
 
@@ -56,6 +20,7 @@ import android.bluetooth.le.AdvertiseData.Builder;
 import android.bluetooth.le.AdvertiseSettings;
 import android.bluetooth.le.BluetoothLeAdvertiser;
 import android.os.ParcelUuid;
+import android.os.Build;
 import android.util.Log;
 import android.util.Pair;
 
@@ -68,7 +33,7 @@ import java.util.ListIterator;
 import java.util.HashMap;
 import java.util.UUID;
 
-public class QtBluetoothLEServer {
+class QtBluetoothLEServer {
     private static final String TAG = "QtBluetoothGattServer";
 
     /* Pointer to the Qt object that "owns" the Java object */
@@ -79,7 +44,7 @@ public class QtBluetoothLEServer {
     private Context qtContext = null;
 
     // Bluetooth members
-    private final BluetoothAdapter mBluetoothAdapter;
+    private BluetoothAdapter mBluetoothAdapter = null;
     private BluetoothManager mBluetoothManager = null;
     private BluetoothGattServer mGattServer = null;
     private BluetoothLeAdvertiser mLeAdvertiser = null;
@@ -89,13 +54,13 @@ public class QtBluetoothLEServer {
 
     private String mRemoteName = "";
     // This function is called from Qt thread
-    public synchronized String remoteName() {
+    synchronized String remoteName() {
         return mRemoteName;
     }
 
     private String mRemoteAddress = "";
     // This function is called from Qt thread
-    public synchronized String remoteAddress() {
+    synchronized String remoteAddress() {
         return mRemoteAddress;
     }
 
@@ -119,12 +84,12 @@ public class QtBluetoothLEServer {
                 this.writes = new ArrayList<Pair<byte[], Integer>>();
         }
         // Returns true if this is a proper entry for given device + target
-        public boolean match(BluetoothDevice device, Object target) {
+        boolean match(BluetoothDevice device, Object target) {
             return remoteDevice.equals(device) && target.equals(target);
         }
-        public final BluetoothDevice remoteDevice; // Device that issued the writes
-        public final Object target; // Characteristic or Descriptor
-        public final List<Pair<byte[], Integer>> writes; // Value, offset
+        final BluetoothDevice remoteDevice; // Device that issued the writes
+        final Object target; // Characteristic or Descriptor
+        final List<Pair<byte[], Integer>> writes; // Value, offset
     }
     private final List<WriteEntry> mPendingPreparedWrites = new ArrayList<>();
 
@@ -187,7 +152,7 @@ public class QtBluetoothLEServer {
             boolean isConnected = false;
         }
 
-        public void insertOrUpdate(BluetoothGattCharacteristic characteristic,
+        void insertOrUpdate(BluetoothGattCharacteristic characteristic,
                               BluetoothDevice device, byte[] newValue)
         {
             if (notificationStore.containsKey(characteristic)) {
@@ -227,7 +192,7 @@ public class QtBluetoothLEServer {
             This function avoids that existing configurations are not acted
             upon when the associated device is not connected.
          */
-        public void markDeviceConnectivity(BluetoothDevice device, boolean isConnected)
+        void markDeviceConnectivity(BluetoothDevice device, boolean isConnected)
         {
             final Iterator<BluetoothGattCharacteristic> keys = notificationStore.keySet().iterator();
             while (keys.hasNext()) {
@@ -282,28 +247,28 @@ public class QtBluetoothLEServer {
             .fromString("00002902-0000-1000-8000-00805f9b34fb");
     ClientCharacteristicManager clientCharacteristicManager = new ClientCharacteristicManager();
 
-    public QtBluetoothLEServer(Context context)
+    QtBluetoothLEServer(Context context)
     {
         qtContext = context;
-        mBluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
-
-        if (mBluetoothAdapter == null || qtContext == null) {
-            Log.w(TAG, "Missing Bluetooth adapter or Qt context. Peripheral role disabled.");
+        if (qtContext == null) {
+            Log.w(TAG, "Missing context object. Peripheral role disabled.");
             return;
         }
 
-        mBluetoothManager = (BluetoothManager) qtContext.getSystemService(Context.BLUETOOTH_SERVICE);
+        mBluetoothManager =
+            (BluetoothManager) qtContext.getSystemService(Context.BLUETOOTH_SERVICE);
         if (mBluetoothManager == null) {
-            Log.w(TAG, "Bluetooth service not available.");
+            Log.w(TAG, "Bluetooth service not available. Peripheral role disabled.");
             return;
         }
 
-        mLeAdvertiser = mBluetoothAdapter.getBluetoothLeAdvertiser();
+        mBluetoothAdapter = mBluetoothManager.getAdapter();
+        if (mBluetoothAdapter == null) {
+            Log.w(TAG, "Missing Bluetooth adapter. Peripheral role disabled.");
+            return;
+        }
 
-        if (!mBluetoothAdapter.isMultipleAdvertisementSupported())
-            Log.w(TAG, "Device does not support Bluetooth Low Energy advertisement.");
-        else
-            Log.w(TAG, "Let's do BTLE Peripheral.");
+        Log.w(TAG, "Let's do BTLE Peripheral.");
     }
 
     // The following functions are synchronized callback handlers. The callbacks
@@ -319,7 +284,7 @@ public class QtBluetoothLEServer {
     //   If each variable would be protected individually, the amount of (nested) locking
     //   would become quite unreasonable
 
-    public synchronized void handleOnConnectionStateChange(BluetoothDevice device,
+    synchronized void handleOnConnectionStateChange(BluetoothDevice device,
                                                            int status, int newState)
     {
         if (mGattServer == null) {
@@ -383,10 +348,10 @@ public class QtBluetoothLEServer {
                 break;
         }
 
-        leServerConnectionStateChange(qtObject, qtErrorCode, qtControllerState);
+        leConnectionStateChange(qtObject, qtErrorCode, qtControllerState);
     }
 
-    public synchronized void handleOnServiceAdded(int status, BluetoothGattService service)
+    synchronized void handleOnServiceAdded(int status, BluetoothGattService service)
     {
         if (mGattServer == null) {
             Log.w(TAG, "Ignoring service addition event, server is disconnected");
@@ -417,7 +382,7 @@ public class QtBluetoothLEServer {
         }
     }
 
-    public synchronized void handleOnCharacteristicReadRequest(BluetoothDevice device,
+    synchronized void handleOnCharacteristicReadRequest(BluetoothDevice device,
                                                    int requestId, int offset,
                                                    BluetoothGattCharacteristic characteristic)
     {
@@ -425,22 +390,25 @@ public class QtBluetoothLEServer {
             Log.w(TAG, "Ignoring characteristic read, server is disconnected");
             return;
         }
-        byte[] dataArray;
+
+        byte[] characteristicData =
+                                ((QtBluetoothGattCharacteristic)characteristic).getLocalValue();
+
         try {
-            dataArray = Arrays.copyOfRange(characteristic.getValue(),
-                                           offset, characteristic.getValue().length);
+            byte[] dataArray = Arrays.copyOfRange(characteristicData,
+                                                  offset, characteristicData.length);
             mGattServer.sendResponse(device, requestId, BluetoothGatt.GATT_SUCCESS,
                                      offset, dataArray);
         } catch (Exception ex) {
             Log.w(TAG, "onCharacteristicReadRequest: " + requestId + " "
-                        + offset + " " + characteristic.getValue().length);
+                        + offset + " " + characteristicData.length);
             ex.printStackTrace();
             mGattServer.sendResponse(device, requestId, BluetoothGatt.GATT_FAILURE,
                                      offset, null);
         }
     }
 
-    public synchronized void handleOnCharacteristicWriteRequest(BluetoothDevice device,
+    synchronized void handleOnCharacteristicWriteRequest(BluetoothDevice device,
                                                    int requestId,
                                                    BluetoothGattCharacteristic characteristic,
                                                    boolean preparedWrite, boolean responseNeeded,
@@ -468,7 +436,7 @@ public class QtBluetoothLEServer {
                       + value.length + ", min: " + minValueLen + ", max: " + maxValueLen);
                 resultStatus = BluetoothGatt.GATT_INVALID_ATTRIBUTE_LENGTH;
             } else if (offset == 0) {
-                characteristic.setValue(value);
+                ((QtBluetoothGattCharacteristic)characteristic).setLocalValue(value);
                 leServerCharacteristicChanged(qtObject, characteristic, value);
                 sendNotificationOrIndication = true;
             } else {
@@ -492,7 +460,7 @@ public class QtBluetoothLEServer {
             sendNotificationsOrIndications(characteristic);
     }
 
-    public synchronized void handleOnDescriptorReadRequest(BluetoothDevice device, int requestId,
+    synchronized void handleOnDescriptorReadRequest(BluetoothDevice device, int requestId,
                                               int offset, BluetoothGattDescriptor descriptor)
     {
         if (mGattServer == null) {
@@ -500,13 +468,14 @@ public class QtBluetoothLEServer {
             return;
         }
 
-        byte[] dataArray = descriptor.getValue();
+        byte[] dataArray = ((QtBluetoothGattDescriptor)descriptor).getLocalValue();
+
         try {
             if (descriptor.getUuid().equals(CLIENT_CHARACTERISTIC_CONFIGURATION_UUID)) {
                 dataArray = clientCharacteristicManager.valueFor(
                             descriptor.getCharacteristic(), device);
                 if (dataArray == null)
-                    dataArray = descriptor.getValue();
+                    dataArray = ((QtBluetoothGattDescriptor)descriptor).getLocalValue();
             }
 
             dataArray = Arrays.copyOfRange(dataArray, offset, dataArray.length);
@@ -521,7 +490,7 @@ public class QtBluetoothLEServer {
         }
     }
 
-    public synchronized void handleOnDescriptorWriteRequest(BluetoothDevice device, int requestId,
+    synchronized void handleOnDescriptorWriteRequest(BluetoothDevice device, int requestId,
                                      BluetoothGattDescriptor descriptor, boolean preparedWrite,
                                      boolean responseNeeded, int offset, byte[] value)
     {
@@ -553,8 +522,7 @@ public class QtBluetoothLEServer {
                                                 descriptor.getCharacteristic(),
                                                 device, value);
                 }
-
-                descriptor.setValue(value);
+                ((QtBluetoothGattDescriptor)descriptor).setLocalValue(value);
                 leServerDescriptorWritten(qtObject, descriptor, value);
             } else {
                 // This should not really happen as per Bluetooth spec
@@ -575,7 +543,7 @@ public class QtBluetoothLEServer {
             mGattServer.sendResponse(device, requestId, resultStatus, offset, value);
     }
 
-    public synchronized void handleOnExecuteWrite(BluetoothDevice device,
+    synchronized void handleOnExecuteWrite(BluetoothDevice device,
                                             int requestId, boolean execute)
     {
         if (mGattServer == null) {
@@ -595,8 +563,8 @@ public class QtBluetoothLEServer {
                 byte[] newValue = null;
                 // The target can be a descriptor or a characteristic
                 byte[] currentValue = (entry.target instanceof BluetoothGattCharacteristic)
-                                ? ((BluetoothGattCharacteristic)entry.target).getValue()
-                                : ((BluetoothGattDescriptor)entry.target).getValue();
+                                    ? ((QtBluetoothGattCharacteristic)entry.target).getLocalValue()
+                                    : ((QtBluetoothGattDescriptor)entry.target).getLocalValue();
 
                 // Iterate writes and apply them to the currentValue in received order
                 for (Pair<byte[], Integer> write : entry.writes) {
@@ -643,11 +611,11 @@ public class QtBluetoothLEServer {
 
                 // Update value and inform the Qt/C++ side on the update
                 if (entry.target instanceof BluetoothGattCharacteristic) {
-                    ((BluetoothGattCharacteristic)entry.target).setValue(newValue);
+                    ((QtBluetoothGattCharacteristic)entry.target).setLocalValue(newValue);
                     leServerCharacteristicChanged(
                         qtObject, (BluetoothGattCharacteristic)entry.target, newValue);
                 } else {
-                    ((BluetoothGattDescriptor)entry.target).setValue(newValue);
+                    ((QtBluetoothGattDescriptor)entry.target).setLocalValue(newValue);
                     leServerDescriptorWritten(
                         qtObject, (BluetoothGattDescriptor)entry.target, newValue);
                 }
@@ -659,7 +627,7 @@ public class QtBluetoothLEServer {
         mGattServer.sendResponse(device, requestId, BluetoothGatt.GATT_SUCCESS, 0, null);
     }
 
-    public synchronized void handleOnMtuChanged(BluetoothDevice device, int mtu)
+    synchronized void handleOnMtuChanged(BluetoothDevice device, int mtu)
     {
         if (mSupportedMtu == mtu)
             return;
@@ -738,12 +706,12 @@ public class QtBluetoothLEServer {
     };
 
     // This function is called from Qt thread
-    public synchronized int mtu() {
+    synchronized int mtu() {
         return mSupportedMtu;
     }
 
     // This function is called from Qt thread
-    public synchronized boolean connectServer()
+    synchronized boolean connectServer()
     {
         if (mGattServer != null)
             return true;
@@ -760,7 +728,7 @@ public class QtBluetoothLEServer {
     }
 
     // This function is called from Qt thread
-    public synchronized void disconnectServer()
+    synchronized void disconnectServer()
     {
         if (mGattServer == null)
             return;
@@ -771,16 +739,29 @@ public class QtBluetoothLEServer {
         mGattServer = null;
 
         mRemoteName = mRemoteAddress = "";
-        leServerConnectionStateChange(qtObject, 0 /*NoError*/, 0 /*QLowEnergyController::UnconnectedState*/);
+        leConnectionStateChange(qtObject, 0 /*NoError*/,
+                                0 /*QLowEnergyController::UnconnectedState*/);
     }
 
     // This function is called from Qt thread
-    public boolean startAdvertising(AdvertiseData advertiseData,
+    boolean startAdvertising(AdvertiseData advertiseData,
                                     AdvertiseData scanResponse,
                                     AdvertiseSettings settings)
     {
-        if (mLeAdvertiser == null)
+        // Check that the bluetooth is on
+        if (mBluetoothAdapter == null || !mBluetoothAdapter.isEnabled()) {
+            Log.w(TAG, "StartAdvertising: Bluetooth not available or offline");
             return false;
+        }
+
+        // According to Android doc this check should always precede the advertiser creation
+        if (mLeAdvertiser == null && mBluetoothAdapter.isMultipleAdvertisementSupported())
+            mLeAdvertiser = mBluetoothAdapter.getBluetoothLeAdvertiser();
+
+        if (mLeAdvertiser == null) {
+            Log.w(TAG, "StartAdvertising: LE advertisement not supported");
+            return false;
+        }
 
         if (!connectServer()) {
             Log.w(TAG, "Server::startAdvertising: Cannot open GATT server");
@@ -794,7 +775,7 @@ public class QtBluetoothLEServer {
     }
 
     // This function is called from Qt thread
-    public void stopAdvertising()
+    void stopAdvertising()
     {
         if (mLeAdvertiser == null)
             return;
@@ -804,7 +785,7 @@ public class QtBluetoothLEServer {
     }
 
     // This function is called from Qt thread
-    public synchronized void addService(BluetoothGattService service)
+    synchronized void addService(BluetoothGattService service)
     {
         if (!connectServer()) {
             Log.w(TAG, "Server::addService: Cannot open GATT server");
@@ -825,6 +806,15 @@ public class QtBluetoothLEServer {
         }
     }
 
+    // API-level < 33
+    @SuppressWarnings("deprecation")
+    private void notifyCharacteristicChange(BluetoothDevice device,
+                                            BluetoothGattCharacteristic characteristic,
+                                            boolean confirm)
+    {
+        mGattServer.notifyCharacteristicChanged(device, characteristic, confirm);
+    }
+
     /*
         Check the client characteristics configuration for the given characteristic
         and sends notifications or indications as per required.
@@ -842,12 +832,25 @@ public class QtBluetoothLEServer {
         //      devices at the same time.
         while (iter.hasNext()) {
             final BluetoothDevice device = iter.next();
-            final byte[] clientCharacteristicConfig = clientCharacteristicManager.valueFor(characteristic, device);
+            final byte[] clientCharacteristicConfig =
+                                      clientCharacteristicManager.valueFor(characteristic, device);
             if (clientCharacteristicConfig != null) {
-                if (Arrays.equals(clientCharacteristicConfig, BluetoothGattDescriptor.ENABLE_NOTIFICATION_VALUE)) {
-                    mGattServer.notifyCharacteristicChanged(device, characteristic, false);
-                } else if (Arrays.equals(clientCharacteristicConfig, BluetoothGattDescriptor.ENABLE_INDICATION_VALUE)) {
-                    mGattServer.notifyCharacteristicChanged(device, characteristic, true);
+                if (Arrays.equals(clientCharacteristicConfig,
+                                  BluetoothGattDescriptor.ENABLE_NOTIFICATION_VALUE)) {
+                    if (Build.VERSION.SDK_INT >= 33) {
+                        mGattServer.notifyCharacteristicChanged(device, characteristic, false,
+                                  ((QtBluetoothGattCharacteristic)characteristic).getLocalValue());
+                    } else {
+                        notifyCharacteristicChange(device, characteristic, false);
+                    }
+                } else if (Arrays.equals(clientCharacteristicConfig,
+                                         BluetoothGattDescriptor.ENABLE_INDICATION_VALUE)) {
+                    if (Build.VERSION.SDK_INT >= 33) {
+                        mGattServer.notifyCharacteristicChanged(device, characteristic, true,
+                                  ((QtBluetoothGattCharacteristic)characteristic).getLocalValue());
+                    } else {
+                        notifyCharacteristicChange(device, characteristic, true);
+                    }
                 }
             }
         }
@@ -855,12 +858,12 @@ public class QtBluetoothLEServer {
 
     /*
         Updates the local database value for the given characteristic with \a charUuid and
-        \a newValue. If notifications for this task are enabled an approproiate notification will
+        \a newValue. If notifications for this task are enabled an appropriate notification will
         be send to the remote client.
 
         This function is called from the Qt thread.
      */
-    public boolean writeCharacteristic(BluetoothGattService service, UUID charUuid, byte[] newValue)
+    boolean writeCharacteristic(BluetoothGattService service, UUID charUuid, byte[] newValue)
     {
         BluetoothGattCharacteristic foundChar = null;
         List<BluetoothGattCharacteristic> charList = service.getCharacteristics();
@@ -892,7 +895,7 @@ public class QtBluetoothLEServer {
 
         synchronized (this) // a value update might be in progress
         {
-            foundChar.setValue(newValue);
+            ((QtBluetoothGattCharacteristic)foundChar).setLocalValue(newValue);
             // Value is updated even if server is not connected, but notifying is not possible
             if (mGattServer != null)
                 sendNotificationsOrIndications(foundChar);
@@ -906,7 +909,7 @@ public class QtBluetoothLEServer {
 
         This function is called from the Qt thread.
      */
-    public boolean writeDescriptor(BluetoothGattService service, UUID charUuid, UUID descUuid,
+    boolean writeDescriptor(BluetoothGattService service, UUID charUuid, UUID descUuid,
                                    byte[] newValue)
     {
         BluetoothGattDescriptor foundDesc = null;
@@ -936,7 +939,7 @@ public class QtBluetoothLEServer {
         // to interpret the server's call as a change of the default value.
         synchronized (this) // a value update might be in progress
         {
-            foundDesc.setValue(newValue);
+            ((QtBluetoothGattDescriptor)foundDesc).setLocalValue(newValue);
         }
 
         return true;
@@ -963,6 +966,7 @@ public class QtBluetoothLEServer {
                 case AdvertiseCallback.ADVERTISE_FAILED_ALREADY_STARTED:
                     return; // ignore -> noop
                 case AdvertiseCallback.ADVERTISE_FAILED_DATA_TOO_LARGE:
+                    Log.e(TAG, "Please reduce size of advertising data.");
                     qtErrorCode = 1;
                     break;
                 case AdvertiseCallback.ADVERTISE_FAILED_FEATURE_UNSUPPORTED:
@@ -982,13 +986,13 @@ public class QtBluetoothLEServer {
         }
     };
 
-    public native void leServerConnectionStateChange(long qtObject, int errorCode, int newState);
-    public native void leMtuChanged(long qtObject, int mtu);
-    public native void leServerAdvertisementError(long qtObject, int status);
-    public native void leServerCharacteristicChanged(long qtObject,
+    native void leConnectionStateChange(long qtObject, int errorCode, int newState);
+    native void leMtuChanged(long qtObject, int mtu);
+    native void leServerAdvertisementError(long qtObject, int status);
+    native void leServerCharacteristicChanged(long qtObject,
                                                      BluetoothGattCharacteristic characteristic,
                                                      byte[] newValue);
-    public native void leServerDescriptorWritten(long qtObject,
+    native void leServerDescriptorWritten(long qtObject,
                                                  BluetoothGattDescriptor descriptor,
                                                  byte[] newValue);
 }
