@@ -19,11 +19,6 @@ QT_BEGIN_NAMESPACE
 
 Q_DECLARE_LOGGING_CATEGORY(QT_BT_BLUEZ)
 
-inline uint qHash(const QBluetoothAddress &address)
-{
-    return ::qHash(address.toUInt64());
-}
-
 QBluetoothLocalDevice::QBluetoothLocalDevice(QObject *parent) :
     QObject(parent),
     d_ptr(new QBluetoothLocalDevicePrivate(this))
@@ -174,6 +169,7 @@ void QBluetoothLocalDevice::requestPairing(const QBluetoothAddress &address, Pai
             if (d_ptr->pairingTarget) {
                 qCDebug(QT_BT_BLUEZ) << "Cancelling pending pairing request to" << d_ptr->pairingTarget->address();
                 QDBusPendingReply<> cancelReply = d_ptr->pairingTarget->CancelPairing();
+                d_ptr->pairingRequestCanceled = true;
                 cancelReply.waitForFinished();
                 delete d_ptr->pairingTarget;
                 d_ptr->pairingTarget = nullptr;
@@ -411,7 +407,7 @@ void QBluetoothLocalDevicePrivate::connectDeviceChanges()
         if (reply.isError())
             return;
 
-        OrgFreedesktopDBusPropertiesInterface *monitor = nullptr;
+        OrgFreedesktopDBusPropertiesInterfaceBluetooth *monitor = nullptr;
 
         ManagedObjectList managedObjectList = reply.value();
         for (ManagedObjectList::const_iterator it = managedObjectList.constBegin(); it != managedObjectList.constEnd(); ++it) {
@@ -427,10 +423,10 @@ void QBluetoothLocalDevicePrivate::connectDeviceChanges()
                 const QVariantMap &ifaceValues = jt.value();
 
                 if (iface == QStringLiteral("org.bluez.Device1")) {
-                    monitor = new OrgFreedesktopDBusPropertiesInterface(QStringLiteral("org.bluez"),
+                    monitor = new OrgFreedesktopDBusPropertiesInterfaceBluetooth(QStringLiteral("org.bluez"),
                                                                         path.path(),
                                                                         QDBusConnection::systemBus(), this);
-                    connect(monitor, &OrgFreedesktopDBusPropertiesInterface::PropertiesChanged,
+                    connect(monitor, &OrgFreedesktopDBusPropertiesInterfaceBluetooth::PropertiesChanged,
                             this, &QBluetoothLocalDevicePrivate::PropertiesChanged);
                     deviceChangeMonitors.insert(path.path(), monitor);
 
@@ -482,10 +478,10 @@ void QBluetoothLocalDevicePrivate::initializeAdapter()
 
     if (adapter) {
         //hook up propertiesChanged for current adapter
-        adapterProperties = new OrgFreedesktopDBusPropertiesInterface(
+        adapterProperties = new OrgFreedesktopDBusPropertiesInterfaceBluetooth(
                 QStringLiteral("org.bluez"), adapter->path(),
                 QDBusConnection::systemBus(), this);
-        connect(adapterProperties, &OrgFreedesktopDBusPropertiesInterface::PropertiesChanged,
+        connect(adapterProperties, &OrgFreedesktopDBusPropertiesInterfaceBluetooth::PropertiesChanged,
                 this, &QBluetoothLocalDevicePrivate::PropertiesChanged);
     }
 }
@@ -532,8 +528,8 @@ void QBluetoothLocalDevicePrivate::PropertiesChanged(const QString &interface,
     } else if (interface == QStringLiteral("org.bluez.Device1")
                && changed_properties.contains(QStringLiteral("Connected"))) {
         // update list of connected devices
-        OrgFreedesktopDBusPropertiesInterface *senderIface =
-                qobject_cast<OrgFreedesktopDBusPropertiesInterface*>(sender());
+        OrgFreedesktopDBusPropertiesInterfaceBluetooth *senderIface =
+                qobject_cast<OrgFreedesktopDBusPropertiesInterfaceBluetooth*>(sender());
         if (!senderIface)
             return;
 
@@ -560,11 +556,11 @@ void QBluetoothLocalDevicePrivate::InterfacesAdded(const QDBusObjectPath &object
         // a new device was added which we need to add to list of known devices
 
         if (objectPathIsForThisDevice(deviceAdapterPath, object_path.path())) {
-            OrgFreedesktopDBusPropertiesInterface *monitor = new OrgFreedesktopDBusPropertiesInterface(
+            OrgFreedesktopDBusPropertiesInterfaceBluetooth *monitor = new OrgFreedesktopDBusPropertiesInterfaceBluetooth(
                                                QStringLiteral("org.bluez"),
                                                object_path.path(),
                                                QDBusConnection::systemBus());
-            connect(monitor, &OrgFreedesktopDBusPropertiesInterface::PropertiesChanged,
+            connect(monitor, &OrgFreedesktopDBusPropertiesInterfaceBluetooth::PropertiesChanged,
                     this, &QBluetoothLocalDevicePrivate::PropertiesChanged);
             deviceChangeMonitors.insert(object_path.path(), monitor);
 
@@ -646,12 +642,19 @@ void QBluetoothLocalDevicePrivate::pairingCompleted(QDBusPendingCallWatcher *wat
 
     if (reply.isError()) {
         qCWarning(QT_BT_BLUEZ) << "Failed to create pairing" << reply.error().name();
-        if (reply.error().name() != QStringLiteral("org.bluez.Error.AuthenticationCanceled"))
+        const bool canceledByUs =
+                (reply.error().name() == QStringLiteral("org.bluez.Error.AuthenticationCanceled"))
+                    && pairingRequestCanceled;
+        if (!canceledByUs)
             emit q->errorOccurred(QBluetoothLocalDevice::PairingError);
+
+        pairingRequestCanceled = false;
         watcher->deleteLater();
         return;
     }
 
+    pairingRequestCanceled = false;
+
     if (adapter) {
         if (!pairingTarget) {
             qCWarning(QT_BT_BLUEZ) << "Pairing target expected but found null pointer.";
