@@ -14,8 +14,10 @@
 #include <QtCore/QLoggingCategory>
 #include <QtCore/private/qfunctions_winrt_p.h>
 #include <QtCore/QDeadlineTimer>
+#include <QtCore/qpointer.h>
 
 #include <functional>
+
 #include <robuffer.h>
 #include <windows.devices.enumeration.h>
 #include <windows.devices.bluetooth.h>
@@ -102,13 +104,14 @@ static QByteArray byteArrayFromGattResult(const ComPtr<IGattReadResult> &gattRes
     return byteArrayFromBuffer(buffer, isWCharString);
 }
 
-static void closeDeviceService(ComPtr<IGattDeviceService> service)
+template <typename T>
+static void closeDeviceService(ComPtr<T> service)
 {
     ComPtr<IClosable> closableService;
     HRESULT hr = service.As(&closableService);
-    RETURN_IF_FAILED("Could not cast service to closable", return);
+    RETURN_IF_FAILED("Could not cast type to closable", return);
     hr = closableService->Close();
-    RETURN_IF_FAILED("Service Close() failed", return);
+    RETURN_IF_FAILED("Close() call failed", return);
 }
 
 class QWinRTLowEnergyServiceHandler : public QObject
@@ -125,12 +128,16 @@ public:
 
     ~QWinRTLowEnergyServiceHandler()
     {
+        if (mAbortRequested)
+            closeDeviceService(mDeviceService);
         qCDebug(QT_BT_WINDOWS) << __FUNCTION__;
     }
 
 public slots:
     void obtainCharList()
     {
+        auto exitCondition = [this]() { return mAbortRequested; };
+
         mIndicateChars.clear();
         qCDebug(QT_BT_WINDOWS) << __FUNCTION__;
         ComPtr<IAsyncOperation<GattCharacteristicsResult *>> characteristicsOp;
@@ -138,7 +145,8 @@ public slots:
         HRESULT hr = mDeviceService->GetCharacteristicsAsync(&characteristicsOp);
         EMIT_WORKER_ERROR_AND_QUIT_IF_FAILED(hr);
         hr = QWinRTFunctions::await(characteristicsOp, characteristicsResult.GetAddressOf(),
-                                    QWinRTFunctions::ProcessMainThreadEvents, 5000);
+                                    QWinRTFunctions::ProcessMainThreadEvents, 5000,
+                                    exitCondition);
         EMIT_WORKER_ERROR_AND_QUIT_IF_FAILED(hr);
         GattCommunicationStatus status;
         hr = characteristicsResult->get_Status(&status);
@@ -156,7 +164,7 @@ public slots:
         EMIT_WORKER_ERROR_AND_QUIT_IF_FAILED(hr);
 
         mCharacteristicsCountToBeDiscovered = characteristicsCount;
-        for (uint i = 0; i < characteristicsCount; ++i) {
+        for (uint i = 0; !mAbortRequested && (i < characteristicsCount); ++i) {
             ComPtr<IGattCharacteristic> characteristic;
             hr = characteristics->GetAt(i, &characteristic);
             if (FAILED(hr)) {
@@ -182,7 +190,9 @@ public slots:
             DEC_CHAR_COUNT_AND_CONTINUE_IF_FAILED(hr, "Could not obtain list of descriptors")
 
             ComPtr<IGattDescriptorsResult> descResult;
-            hr = QWinRTFunctions::await(descAsyncOp, descResult.GetAddressOf());
+            hr = QWinRTFunctions::await(descAsyncOp, descResult.GetAddressOf(),
+                                        QWinRTFunctions::ProcessMainThreadEvents, 5000,
+                                        exitCondition);
             DEC_CHAR_COUNT_AND_CONTINUE_IF_FAILED(hr, "Could not obtain descriptor read result")
 
             quint16 handle;
@@ -211,7 +221,9 @@ public slots:
                                                                  &readOp);
                 DEC_CHAR_COUNT_AND_CONTINUE_IF_FAILED(hr, "Could not read characteristic")
                 ComPtr<IGattReadResult> readResult;
-                hr = QWinRTFunctions::await(readOp, readResult.GetAddressOf());
+                hr = QWinRTFunctions::await(readOp, readResult.GetAddressOf(),
+                                            QWinRTFunctions::ProcessMainThreadEvents, 5000,
+                                            exitCondition);
                 DEC_CHAR_COUNT_AND_CONTINUE_IF_FAILED(hr,
                                                       "Could not obtain characteristic read result")
                 if (!readResult)
@@ -237,7 +249,7 @@ public slots:
             uint descriptorCount;
             hr = descriptors->get_Size(&descriptorCount);
             DEC_CHAR_COUNT_AND_CONTINUE_IF_FAILED(hr, "Could not obtain list of descriptors' size")
-            for (uint j = 0; j < descriptorCount; ++j) {
+            for (uint j = 0; !mAbortRequested && (j < descriptorCount); ++j) {
                 QLowEnergyServicePrivate::DescData descData;
                 ComPtr<IGattDescriptor> descriptor;
                 hr = descriptors->GetAt(j, &descriptor);
@@ -257,7 +269,9 @@ public slots:
                                 &readOp);
                         WARN_AND_CONTINUE_IF_FAILED(hr, "Could not read descriptor value")
                         ComPtr<IClientCharConfigDescriptorResult> readResult;
-                        hr = QWinRTFunctions::await(readOp, readResult.GetAddressOf());
+                        hr = QWinRTFunctions::await(readOp, readResult.GetAddressOf(),
+                                                    QWinRTFunctions::ProcessMainThreadEvents, 5000,
+                                                    exitCondition);
                         WARN_AND_CONTINUE_IF_FAILED(hr, "Could not await descriptor read result")
                         GattClientCharacteristicConfigurationDescriptorValue value;
                         hr = readResult->get_ClientCharacteristicConfigurationDescriptor(&value);
@@ -289,12 +303,11 @@ public slots:
                                                                      &readOp);
                         WARN_AND_CONTINUE_IF_FAILED(hr, "Could not read descriptor value")
                         ComPtr<IGattReadResult> readResult;
-                        hr = QWinRTFunctions::await(readOp, readResult.GetAddressOf());
-                        WARN_AND_CONTINUE_IF_FAILED(hr, "Could await descriptor read result")
-                        if (descData.uuid == QBluetoothUuid::DescriptorType::CharacteristicUserDescription)
-                            descData.value = byteArrayFromGattResult(readResult, true);
-                        else
-                            descData.value = byteArrayFromGattResult(readResult);
+                        hr = QWinRTFunctions::await(readOp, readResult.GetAddressOf(),
+                                                    QWinRTFunctions::ProcessMainThreadEvents, 5000,
+                                                    exitCondition);
+                        WARN_AND_CONTINUE_IF_FAILED(hr, "Could not await descriptor read result")
+                        descData.value = byteArrayFromGattResult(readResult);
                     }
                 }
                 charData.descriptorList.insert(descHandle, descData);
@@ -306,8 +319,13 @@ public slots:
         checkAllCharacteristicsDiscovered();
     }
 
+    void setAbortRequested()
+    {
+        mAbortRequested = true;
+    }
+
 private:
-    bool checkAllCharacteristicsDiscovered();
+    void checkAllCharacteristicsDiscovered();
     void emitErrorAndQuitThread(HRESULT hr);
     void emitErrorAndQuitThread(const QString &error);
 
@@ -320,6 +338,7 @@ public:
     quint16 mStartHandle = 0;
     quint16 mEndHandle = 0;
     QList<QBluetoothUuid> mIndicateChars;
+    bool mAbortRequested = false;
 
 signals:
     void charListObtained(const QBluetoothUuid &service,
@@ -329,16 +348,13 @@ signals:
     void errorOccured(const QString &error);
 };
 
-bool QWinRTLowEnergyServiceHandler::checkAllCharacteristicsDiscovered()
+void QWinRTLowEnergyServiceHandler::checkAllCharacteristicsDiscovered()
 {
-    if (mCharacteristicsCountToBeDiscovered == 0) {
+    if (!mAbortRequested && (mCharacteristicsCountToBeDiscovered == 0)) {
         emit charListObtained(mService, mCharacteristicList, mIndicateChars,
                               mStartHandle, mEndHandle);
-        QThread::currentThread()->quit();
-        return true;
     }
-
-    return false;
+    QThread::currentThread()->quit();
 }
 
 void QWinRTLowEnergyServiceHandler::emitErrorAndQuitThread(HRESULT hr)
@@ -348,6 +364,7 @@ void QWinRTLowEnergyServiceHandler::emitErrorAndQuitThread(HRESULT hr)
 
 void QWinRTLowEnergyServiceHandler::emitErrorAndQuitThread(const QString &error)
 {
+    mAbortRequested = true; // so that the service is closed during cleanup
     emit errorOccured(error);
     QThread::currentThread()->quit();
 }
@@ -911,7 +928,7 @@ HRESULT QLowEnergyControllerPrivateWinRT::onMtuChange(IGattSession *session, IIn
 {
     qCDebug(QT_BT_WINDOWS) << __FUNCTION__;
     if (session != mGattSession.Get()) {
-        qCWarning(QT_BT_WINDOWS) << "Got MTU changed event for wrong GattSession.";
+        qCWarning(QT_BT_WINDOWS) << "Got MTU changed event for wrong or outdated GattSession.";
         return S_OK;
     }
 
@@ -1139,6 +1156,12 @@ HRESULT QLowEnergyControllerPrivateWinRT::onServiceDiscoveryFinished(ABI::Window
 
 void QLowEnergyControllerPrivateWinRT::clearAllServices()
 {
+    // These services will be closed in the respective
+    // QWinRTLowEnergyServiceHandler workers (in background threads).
+    for (auto &uuid : m_requestDetailsServiceUuids)
+        m_openedServices.remove(uuid);
+    m_requestDetailsServiceUuids.clear();
+
     for (auto service : m_openedServices) {
         closeDeviceService(service);
     }
@@ -1205,6 +1228,10 @@ void QLowEnergyControllerPrivateWinRT::discoverServiceDetailsHelper(
     };
     //update service data
     QSharedPointer<QLowEnergyServicePrivate> pointer = serviceList.value(service);
+    if (!pointer) {
+        qCDebug(QT_BT_WINDOWS) << "Device was disconnected while doing service discovery";
+        return;
+    }
     qCDebug(QT_BT_WINDOWS_SERVICE_THREAD) << __FUNCTION__ << "Changing service pointer from thread"
                                           << QThread::currentThread();
     pointer->setState(QLowEnergyService::RemoteServiceDiscovering);
@@ -1276,11 +1303,14 @@ void QLowEnergyControllerPrivateWinRT::discoverServiceDetailsHelper(
 
     QWinRTLowEnergyServiceHandler *worker =
             new QWinRTLowEnergyServiceHandler(service, deviceService3, mode);
+    m_requestDetailsServiceUuids.insert(service);
     QThread *thread = new QThread;
     worker->moveToThread(thread);
     connect(thread, &QThread::started, worker, &QWinRTLowEnergyServiceHandler::obtainCharList);
     connect(thread, &QThread::finished, worker, &QObject::deleteLater);
     connect(worker, &QObject::destroyed, thread, &QObject::deleteLater);
+    connect(this, &QLowEnergyControllerPrivateWinRT::abortConnection,
+            worker, &QWinRTLowEnergyServiceHandler::setAbortRequested);
     connect(worker, &QWinRTLowEnergyServiceHandler::errorOccured,
             this, &QLowEnergyControllerPrivateWinRT::handleServiceHandlerError);
     connect(worker, &QWinRTLowEnergyServiceHandler::charListObtained, this,
@@ -1292,6 +1322,7 @@ void QLowEnergyControllerPrivateWinRT::discoverServiceDetailsHelper(
                     << "Discovery complete for unknown service:" << service.toString();
             return;
         }
+        m_requestDetailsServiceUuids.remove(service);
 
         QSharedPointer<QLowEnergyServicePrivate> pointer = serviceList.value(service);
         pointer->startHandle = startHandle;
