@@ -1,30 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 Research In Motion
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtNfc module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 Research In Motion
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QtTest/QtTest>
 
@@ -136,16 +111,12 @@ QString tst_QNdefNfcSmartPosterRecord::getTitle(const QString& locale)
 
 void tst_QNdefNfcSmartPosterRecord::checkLocale(const QNdefNfcSmartPosterRecord& record, const QStringList& localeList)
 {
-    QList<QString> locales = _textRecords.keys();
-
-    for (int i = 0; i < locales.size(); i++) {
-        if (localeList.contains(locales[i])) {
-            QVERIFY(record.hasTitle(locales[i]));
-        }
-
-        else {
-            QVERIFY(!record.hasTitle(locales[i]));
-        }
+    for (auto it = _textRecords.cbegin(), end = _textRecords.cend(); it != end; ++it) {
+        const QString &locale = it.key();
+        if (localeList.contains(locale))
+            QVERIFY(record.hasTitle(locale));
+        else
+            QVERIFY(!record.hasTitle(locale));
     }
 
     if (localeList.empty()) {
@@ -428,7 +399,7 @@ void tst_QNdefNfcSmartPosterRecord::tst_size()
 {
     QNdefNfcSmartPosterRecord record;
     QVERIFY(!record.hasSize());
-    QCOMPARE(record.size(), 0);
+    QCOMPARE(record.size(), 0U);
 
     quint32 size = 1024;
     record.setSize(size);
@@ -561,7 +532,7 @@ void tst_QNdefNfcSmartPosterRecord::tst_downcast()
     QByteArray spPayload = sprecord.payload();
 
     // Check length is longer on the base
-    QVERIFY(basePayload.length() > record.payload().length());
+    QVERIFY(basePayload.size() > record.payload().size());
 
     // Check the payloads are the same
     QCOMPARE(basePayload, spPayload);
